//<c>   Copyright 1990-2000 by Gerry J. Danen; all rights reserved.

//<t>   GST calculator

#include"gd_tools.h"
#include"gd_extrn.h"
#include"gd_scrn.h"
#include"gd_wn.h"
#include"gd_kb.h"
#include<math.h>
#include<stdarg.h>


#define pgVERS_ "V2.10"
#define pgTITL_ "GST Calculator"
#define pgCOPR_ "1990"


UCHAR   str[81], line[81], *l ;
int     err = FALSE ;
double  gst_pct = 7.0 ;


GD_DSP  write_scr_help_data[] = {
         4,  4, RED, WHITE,     "The first line shows your amount with the assumption that it is a net",
         5,  4, RED, WHITE,     "amount and that you wish to add GST.",
         6,  4, RED, WHITE,     "The GST and the GST-in amount are calculated.",
         8,  4, RED, WHITE,     "The second line shows your amount and the program assumes that it is a",
         9,  4, RED, WHITE,     "GST-in amount and that you wish to remove the GST.",
        10,  4, RED, WHITE,     "The GST and the net amount are calculated.",
        15, 11, RED, WHITE,     "Amounts must be in the range of 0.00 to 20,000,000.00",
        22, 22, RED, WHITE,     "Suggestions for improvement are welcome!",
         0,  0, 0, 0, "" };


void    help( void )
{
    sc_save_video_state();
    box( RED, WHITE, 1,1, 25,80, YES, 1 );
    sc_centered( 1, BR_WHITE, RED, " %s ", pgm_title );
    sc_write_data( write_scr_help_data );
    sc_centered( 25, WHITE, RED, " %s ", "Press any key to return..." );
    CursOn();
    kb_getkey_();
    sc_restore_video_state();
}


void    show_line( UCHAR *fmt, ... )
{
    va_list     arg_ptr ;

    va_start( arg_ptr, fmt );
    vsprintf( ErrMsg_buf_, fmt, arg_ptr );
    va_end( arg_ptr );
    sc_scroll( 1, vid.normal_attr, 10,1, 24,80, SCROLL_UP );
    sc_msg_at( 24,1, "%s", ErrMsg_buf_ );
}


BOOL    get_line( void )
{
    int         gotline ;

    do  {
        gotline = TRUE;
        EnterText( line, 20, FALSE, 0, " Amount" );
        st_trim( line );
        if ( st_len(line) < 1 )
            return( FALSE );
        st_remove_spaces( line );
        StrDelChar( line, ',' );
        StrDelChar( line, '$' );
        StrDelChar( line, '/' );
        l = line;

        if ( st_eq( "?", l ) || st_eq( "help", l ) )
        {
            help();
            gotline = FALSE;
        }
    }   while ( gotline == FALSE );
    show_line( " " );
    return( TRUE );
}


double  get_number( void )
{
    double      result, r, n_output ;
    int         sign ;

    err = FALSE ;
    result = (double) 0.0 ;
    sign = 1;
    if ( *l == '-' )
        sign = -1;
    if ( (*l == '-') || (*l == '+') )
        l++;
    if ( ((*l < '0') || (*l > '9')) && (*l != '.') )
    {
        err = TRUE ;
        return( 0.0 );
    }
    while ((*l >= '0') && (*l <= '9'))
    {
        result = result * 10 + *l++ - '0';
    }
    if ( *l == '.' )
    {
        l++;
        r = .1;
        while ((*l >= '0') && (*l <= '9'))
        {
            result += (*l++ - '0') * r;
            r /= 10;
        }
    }
    n_output = dbl_round( (sign * result), 2 );
    if ( n_output < 0.0 || n_output > 20000000 )        // 0...20,000,000.00
    {
        err = TRUE ;
        return( 0.0 );
    }
    return( n_output );
}


void    process_line( void )
{
    double      num, gst, namt ;
    UCHAR       s_gst[20], s_amt[20], s_inp[20] ;

    if ( strlen(line) )
    {
        num  = get_number();
        if ( err )
        {
            show_line( "?input error...  enter HELP for instructions!" );
            sn_beep();
        }
        else
        {
            gst   = dbl_round( (num * gst_pct / 100.0), 2 );
            namt  = num + gst ;
            PicAmount( dbl_2_long(num , 2, YES), s_inp, "ZZZ,ZZZ,ZZ9.99" );
            PicAmount( dbl_2_long(gst , 2, YES), s_gst,   "Z,ZZZ,ZZ9.99" );
            PicAmount( dbl_2_long(namt, 2, YES), s_amt, "ZZZ,ZZZ,ZZ9.99" );
            show_line( "%25s plus %s GST = %s", s_inp, s_gst, s_amt );

            gst  = dbl_round( (num * gst_pct / (100.0+gst_pct)), 2 );
            namt = num - gst ;
            PicAmount( dbl_2_long(num , 2, YES), s_inp, "ZZZ,ZZZ,ZZ9.99" );
            PicAmount( dbl_2_long(gst , 2, YES), s_gst,   "Z,ZZZ,ZZ9.99" );
            PicAmount( dbl_2_long(namt, 2, YES), s_amt, "ZZZ,ZZZ,ZZ9.99" );
            show_line( "%25s less %s GST = %s", s_inp, s_gst, s_amt );
        }
    }
}


void    main( void )
{
    fs_init( FALSE, SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    Allow_Break();                              // allow ^Break to abort
    NewScreen();
    sc_centered( 4, -1,-1, "%s", pgm_copyrite );
    sc_centered( 8, 99,99, "%s", " To quit, just press [Enter] when asked for the amount " );

    PicAmount( dbl_2_long(gst_pct, 2, YES), str, "ZZZ,ZZ9.99" );
    st_trim( str );
    show_line( "GST = %s%%", str );

    for (;;)
    {
        if ( ! get_line() )
        {
            show_line( " " );
            ClearEOS();
            gd_exit( 0, "" );
        }
        process_line();
    }
}
